==> interzone-1.0.11.tar.gz: lib/ModeratorSites.dart <==
import 'dart:async';
import 'dart:typed_data';
import 'world.dart';

class ModeratorSites {
  //init site from startup list
  //or join site
  interzoneDisk? disk;
  Map<String, world> _sites = {};
  bool get _dirty {
    for (var s in _sites.values) {
      if (s.isDirty) return true;
    }
    return false;
  }

  void unDirty() {
    for (var s in _sites.values) {
      s.unDirty();
    }
  }

  void init(interzoneDisk _io) {
    disk = _io;
    //read site specific things
  }

  List<String> siteKeys = [];
  String homeSite = '';
  String lastPushedCID = '';
  int lastPushedCIDByteLength = 0;

  List<world> get allSites => _sites.values.toList();

  Future<bool> followSite(
      String siteName, String url, String nick, String avatar) async {
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    if (_sites[siteName] != null) return true;

    if (nick.isEmpty) nick = _sites[homeSite]!.currentNick;
    if (avatar.isEmpty) avatar = _sites[homeSite]!.currentAvatar;

    await disk!.init(siteName + 'Data', false);
    final world iz = world();
    iz.appSplashImage = siteName;
    return iz.init(siteName, url, nick, avatar, disk!).then((value) {
      if (value == null) return false;
      _sites[siteName] = iz;
      siteKeys.add(siteName);
      if (homeSite.isEmpty) homeSite = siteName;
      ModeratorEntrySet _es = ModeratorEntrySet();

      _sites.forEach((key, value) {
        value.messages.all.forEach((element) {
          element.siteUrl = key;
          _es.add(element);
        });
      });
      _combinedEntrySetCache = _es;

      return true;
    });
  }

  bool switchHomeSite(String siteName, String url) {
    //gets url
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    if (homeSite == siteName) return true;
    if (!siteKeys.contains(siteName)) return false;
    homeSite = siteName;
    //TODO flush previous homesite
    _sites[siteName]!.ping();
    return true;
  }

  bool ping() {
    allSites.forEach((element) {
      element.ping();
    });
    return true;
  }

  world? findSite(String site) => _sites[site];
  world? get findHomeSite => findSite(homeSite);
  String get nick {
    if (findHomeSite != null) return findHomeSite!.currentNick;
    return 'deadHomeSitebeef';
  }

  String get avatar {
    if (findHomeSite != null) return findHomeSite!.currentAvatar;
    return '🐷';
  }

  ModeratorEntrySet _combinedEntrySetCache = ModeratorEntrySet();

  ModeratorEntrySet get combinedEntrySet {
    if (!_dirty && _combinedEntrySetCache.all.isNotEmpty) {
      return _combinedEntrySetCache;
    }
    ModeratorEntrySet _es = ModeratorEntrySet();
    _sites.forEach((key, value) {
      value.messages.all.forEach((element) {
        element.siteUrl = key;
        _es.add(element);
      });
    });
    _combinedEntrySetCache = _es;
    unDirty();
    return _combinedEntrySetCache;
  }

  ModeratorEntrySet _latestThreadsEntrySetCache = ModeratorEntrySet();

  ModeratorEntrySet get latestThreadsEntrySet {
    //find stuff with updated stuff possibly
    if (!_dirty && _latestThreadsEntrySetCache.all.isNotEmpty) {
      return _latestThreadsEntrySetCache;
    }
    Map<int, int> _sorta = {};
    Map<int, ModeratorEntry> _all = {};
    int cou = 0;
    Set<String> _dups = {};
    _sites.forEach((key, value) {
      value.LatestThreads.posts.forEach((ModeratorEntry element) {
        if (_dups.contains(element.cleanedBody)) return;
        _dups.add(element.cleanedBody);

        element.siteUrl = key;
        _all[cou] = element;
        _sorta[cou] = element.perceivedTimestamp;
        cou++;
      });
    });
    //sort by keys
    var tl = _sorta.keys.toList();
    tl.sort();
    ModeratorEntrySet _es = ModeratorEntrySet();
    for (var i in tl) {
      _es.add(_all[i]!);
    }
    _latestThreadsEntrySetCache = _es;
    //recalc thread shortcuts
    _tbsCache.clear;
    unDirty();
    return _latestThreadsEntrySetCache;
  }

  ModeratorEntrySet taggedAs(String tag) =>
      ModeratorEntrySet.fromList(latestThreadsEntrySet.taggedAs(tag));
  ModeratorEntrySet taggedAsWithReplies(String shortLink) {
    final t = threadByShortLink(shortLink);
    if (t.isEmpty) return ModeratorEntrySet();
    final tag = t.first.tags.first;

    ModeratorEntrySet _l = ModeratorEntrySet();
    taggedAs(tag).all.forEach((element) {
      _l.add(element);
      threadByShortLink(element.shortLink).forEach((e2) {
        if (e2.flags.isReply) {
          _l.add(e2);
        }
      });
    });
    return _l;
  }

  ModeratorEntrySet AuthoredByWithReplies(String shortLink) {
    final t = threadByShortLink(shortLink);
    if (t.isEmpty) return ModeratorEntrySet();
    final nick = t.first.nick;

    ModeratorEntrySet _l = ModeratorEntrySet();
    authoredBy(nick).all.forEach((element) {
      _l.add(element);
      threadByShortLink(element.shortLink).forEach((e2) {
        if (e2.flags.isReply) {
          _l.add(e2);
        }
      });
    });
    return _l;
  }

  ModeratorEntrySet mentioning(String tag) =>
      ModeratorEntrySet.fromList(latestThreadsEntrySet.mentioning(tag));
  ModeratorEntrySet authoredBy(String tag) =>
      ModeratorEntrySet.fromList(latestThreadsEntrySet.authoredBy(tag));

  Map<String, List<ModeratorEntry>> _tbsCache = {};

  List<ModeratorEntry> threadByShortLink(String shortLink) {
    if (_tbsCache[shortLink] != null) return _tbsCache[shortLink]!;
    Set<ModeratorEntry> _ll = {};
    for (var w in allSites) {
      final s = w.threadByShortLink(shortLink);
      if (s.isNotEmpty) {
        _ll.addAll(s);
        break;
      }
    }
    _tbsCache[shortLink] = _ll.toList();
    return _ll.toList();
  }

  List<ModeratorEntry> unreadThreadByShortLink(String shortLink) {
    return latestThreadsEntrySet.threadByShortLink(shortLink).toList();
  }

  ModeratorEntrySet? siteEntrySet(String site) {
    ModeratorEntrySet _es = ModeratorEntrySet();
    if (_sites[site] == null) return _es;
    return _sites[site]?.messages;
  }

  Future<bool> setNickAvatar(String nick, String avatar, String site) async {
    if (site.isEmpty) site = homeSite;
    return await _sites[site]!.setUserNickAndImage(nick, avatar, Uint8List(0));
  }

  Future<bool> setIpfsCredentials(
      String url, String k1, String k2, String site) async {
    if (site.isEmpty) {
      _sites.values.forEach((element) {
        element.setIpfsCredentials(k1, k2, url);
      });
      return true;
    }
    return await _sites[site]!.setIpfsCredentials(k1, k2, '');
  }

  Future<bool> unfollowSite(String siteName, String url, String nick) async {
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    //todo flush caches etc
    _sites[siteName]!.terminate();
    _sites.remove(siteName);
    siteKeys.remove(siteName);
    if (homeSite == siteName) {
      homeSite = siteKeys.first;
    }

    ModeratorEntrySet _es = ModeratorEntrySet();
    _sites.forEach((key, value) {
      value.messages.all.forEach((element) {
        element.siteUrl = key;
        _es.add(element);
      });
    });
    _combinedEntrySetCache = _es;

    return true;
  }

  /*Future<bool> deleteEntry(String shortLink) async {
    if (_sites[homeSite] == null) return false;
    _sites[homeSite]!.deleteEntry(shortLink);
    return true;
  }*/

  Future<bool> dropEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = _sites[surl]!.dropEntry(shortLink);
    if (li == null) return false;
    _sites[surl]!.dropEntryOmni(shortLink);
    _tbsCache.remove(shortLink);
    return true;
  }

  bool markSeenOmni(String shortLink) {
    //hack to update cache
    if (_latestThreadsEntrySetCache.markSeenByShortLink(shortLink)) {
      ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
      if (target == null) return false;
      final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
      final li = _sites[surl]!.markSeenOmni(shortLink);
      if (li == null) return false;
    }

    _tbsCache.remove(shortLink);
    return true;
  }

  bool isSeenOmni(String shortLink) {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    return _sites[surl]!.isSeenOmni(shortLink);
  }

  bool peekThreadOmni(String shortLink) {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = _sites[surl]!.peekThreadOmni(shortLink);
    if (li == null) return false;
    return true;
  }

  Future<bool> muteAuthor(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    //TODO add OMNIdrops

    return await _sites[surl]!.muteAuthor(target);
  }

  Future<bool> shareEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = await _sites[surl]!.shareEntry(target);
    if (li == null) return false;
    return true;
  }

  Future<bool> likeEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final li = _sites[target.siteUrl]!.likeEntry(target);
    if (li == null) return false;
    return true;
  }

  Future<ModeratorEntry?> postEntry(
      String shortLink, String text, Uint8List attachment) async {
    if (_sites[homeSite] == null) return null;

    final pe = await _sites[homeSite]!
        .newThread(homeSite, text, attachment, '', shortLink, '');
    if (pe != null && pe.isEmpty) return null;
    return pe!.first;
  }

  Future<List<ModeratorEntry>?> replyEntry(
      String shortLink, String replyText, String ppShortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return null;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;

    return _sites[surl]!
        .newReply(replyText, target.shortLink, ppShortLink)
        .then((value) {
      if (value != null && value!.isNotEmpty) {
        _tbsCache.remove(shortLink);
      }
      return value;
    });
  }

  Future<List<ModeratorEntry>?> refreshThread(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    //if thread missing try to find on serv
    //if (target == null) return false;
    //final surl = target?.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final surl = homeSite;
    final r = await _sites[surl]!.refreshThreadViaHTTP(shortLink);
    if (r == null) return r;
    _tbsCache.clear();
    return r;
  }

  Future<List<ModeratorEntry>?> refreshTagged(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    //if thread missing try to find on serv
    //if (target == null) return false;
    //final surl = target?.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final surl = homeSite;
    final r = await _sites[surl]!.refreshTaggedViaHTTP(shortLink);
    if (r == null) return r;
    _tbsCache.clear();
    return r;
  }

  Future<List<ModeratorEntry>?> refreshAuthor(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    //if thread missing try to find on serv
    //if (target == null) return false;
    //final surl = target?.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final surl = homeSite;
    final r = await _sites[surl]!.refreshAuthorViaHTTP(shortLink);
    if (r == null) return r;
    _tbsCache.clear();
    return r;
  }

  Future<Uint8List> pullIPFScid(String cid, int xx) async {
    if (cid.isEmpty) {
      return Uint8List(0);
    }
    return _sites[homeSite]!.xxC.pullIPFScid(cid, xx);
  }

  ModeratorEntry? findModeratorEntryByShortLink(String shortLink) {
    for (var i in _sites.values) {
      final t = i.findModeratorEntryByShortLink(shortLink);
      if (t != null) return t;
    }
    return null;
  }

  Future<bool> get refreshAllViaHTTP async {
    allSites.forEach((element) async {
      await element.refreshViaHTTP();
    });

    return true;
  }

  List<int> get stats {
    final s = [
      0, //IOreceived
      0, //IOsent
      0,
      0,
      0, //memUsed
      0, //audience
      0, 0, 0, 0, 0, 0, 0
    ];
    //combined stats for each followed server
    allSites.forEach((element) async {
      var ii = 0;
      for (var i in element.stats) {
        s[ii] = s[ii] + i;
        ii++;
      }
    });
    return s;
  }

  Uint8List get katamari {
    return _sites[homeSite]!.katamari;
  }
}

==> interzone-1.0.13.tar.gz: lib/ModeratorSites.dart <==
import 'dart:async';
import 'dart:typed_data';
import 'world.dart';

class ModeratorSites {
  //init site from startup list
  //or join site
  interzoneDisk? disk;
  Map<String, world> _sites = {};
  bool get _dirty {
    for (var s in _sites.values) {
      if (s.isDirty) return true;
    }
    return false;
  }

  void unDirty() {
    for (var s in _sites.values) {
      s.unDirty();
    }
  }

  void init(interzoneDisk _io) {
    disk = _io;
    //read site specific things
  }

  List<String> siteKeys = [];
  String homeSite = '';
  String lastPushedCID = '';
  int lastPushedCIDByteLength = 0;

  List<world> get allSites => _sites.values.toList();

  Future<bool> followSite(
      String siteName, String url, String nick, String avatar) async {
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    if (_sites[siteName] != null) return true;

    if (nick.isEmpty) nick = _sites[homeSite]!.currentNick;
    if (avatar.isEmpty) avatar = _sites[homeSite]!.currentAvatar;

    await disk!.init(siteName + 'Data', false);
    final world iz = world();
    iz.appSplashImage = siteName;
    return await iz.init(siteName, url, nick, avatar, disk!).then((value) {
      if (value == null) return false;
      _sites[siteName] = iz;
      siteKeys.add(siteName);
      if (homeSite.isEmpty) homeSite = siteName;
      ModeratorEntrySet _es = ModeratorEntrySet();

      _sites.forEach((key, value) {
        value.messages.all.forEach((element) {
          element.siteUrl = key;
          _es.add(element);
        });
      });
      _combinedEntrySetCache = _es;

      return true;
    });
  }

  bool switchHomeSite(String siteName, String url) {
    //gets url
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    if (homeSite == siteName) return true;
    if (!siteKeys.contains(siteName)) return false;
    homeSite = siteName;
    //TODO flush previous homesite
    _sites[siteName]!.ping();
    return true;
  }

  bool ping() {
    allSites.forEach((element) {
      element.ping();
    });
    return true;
  }

  world? findSite(String site) => _sites[site];
  world? get findHomeSite => findSite(homeSite);
  String get nick {
    if (findHomeSite != null) return findHomeSite!.currentNick;
    return 'deadHomeSitebeef';
  }

  String get avatar {
    if (findHomeSite != null) return findHomeSite!.currentAvatar;
    return '🐷';
  }

  ModeratorEntrySet _combinedEntrySetCache = ModeratorEntrySet();

  ModeratorEntrySet get combinedEntrySet {
    if (!_dirty && _combinedEntrySetCache.all.isNotEmpty) {
      return _combinedEntrySetCache;
    }
    ModeratorEntrySet _es = ModeratorEntrySet();
    _sites.forEach((key, value) {
      value.messages.all.forEach((element) {
        element.siteUrl = key;
        _es.add(element);
      });
    });
    _combinedEntrySetCache = _es;
    unDirty();
    return _combinedEntrySetCache;
  }

  ModeratorEntrySet _latestThreadsEntrySetCache = ModeratorEntrySet();

  ModeratorEntrySet get latestThreadsEntrySet {
    //find stuff with updated stuff possibly
    if (!_dirty && _latestThreadsEntrySetCache.all.isNotEmpty) {
      return _latestThreadsEntrySetCache;
    }
    Map<int, int> _sorta = {};
    Map<int, ModeratorEntry> _all = {};
    int cou = 0;
    Set<String> _dups = {};
    _sites.forEach((key, value) {
      value.LatestThreads.posts.forEach((ModeratorEntry element) {
        if (_dups.contains(element.cleanedBody)) return;
        _dups.add(element.cleanedBody);

        element.siteUrl = key;
        _all[cou] = element;
        _sorta[cou] = element.perceivedTimestamp;
        cou++;
      });
    });
    //sort by keys
    var tl = _sorta.keys.toList();
    tl.sort();
    ModeratorEntrySet _es = ModeratorEntrySet();
    for (var i in tl) {
      _es.add(_all[i]!);
    }
    _latestThreadsEntrySetCache = _es;
    //recalc thread shortcuts
    _tbsCache.clear;
    unDirty();
    return _latestThreadsEntrySetCache;
  }

  ModeratorEntrySet taggedAs(String tag) =>
      ModeratorEntrySet.fromList(latestThreadsEntrySet.taggedAs(tag));
  ModeratorEntrySet taggedAsWithReplies(String shortLink) {
    final t = threadByShortLink(shortLink);
    if (t.isEmpty) return ModeratorEntrySet();
    final tag = t.first.tags.first;

    ModeratorEntrySet _l = ModeratorEntrySet();
    taggedAs(tag).all.forEach((element) {
      _l.add(element);
      threadByShortLink(element.shortLink).forEach((e2) {
        if (e2.flags.isReply) {
          _l.add(e2);
        }
      });
    });
    return _l;
  }

  ModeratorEntrySet AuthoredByWithReplies(String shortLink) {
    final t = threadByShortLink(shortLink);
    if (t.isEmpty) return ModeratorEntrySet();
    final nick = t.first.nick;

    ModeratorEntrySet _l = ModeratorEntrySet();
    authoredBy(nick).all.forEach((element) {
      _l.add(element);
      threadByShortLink(element.shortLink).forEach((e2) {
        if (e2.flags.isReply) {
          _l.add(e2);
        }
      });
    });
    return _l;
  }

  ModeratorEntrySet mentioning(String tag) =>
      ModeratorEntrySet.fromList(latestThreadsEntrySet.mentioning(tag));
  ModeratorEntrySet authoredBy(String tag) =>
      ModeratorEntrySet.fromList(latestThreadsEntrySet.authoredBy(tag));

  Map<String, List<ModeratorEntry>> _tbsCache = {};

  List<ModeratorEntry> threadByShortLink(String shortLink) {
    if (_tbsCache[shortLink] != null) return _tbsCache[shortLink]!;
    Set<ModeratorEntry> _ll = {};
    for (var w in allSites) {
      final s = w.threadByShortLink(shortLink);
      if (s.isNotEmpty) {
        _ll.addAll(s);
        break;
      }
    }
    _tbsCache[shortLink] = _ll.toList();
    return _ll.toList();
  }

  List<ModeratorEntry> unreadThreadByShortLink(String shortLink) {
    return latestThreadsEntrySet.threadByShortLink(shortLink).toList();
  }

  ModeratorEntrySet? siteEntrySet(String site) {
    ModeratorEntrySet _es = ModeratorEntrySet();
    if (_sites[site] == null) return _es;
    return _sites[site]?.messages;
  }

  Future<bool> setNickAvatar(String nick, String avatar, String site) async {
    if (site.isEmpty) site = homeSite;
    return await _sites[site]!.setUserNickAndImage(nick, avatar, Uint8List(0));
  }

  Future<bool> setIpfsCredentials(
      String url, String k1, String k2, String site) async {
    if (site.isEmpty) {
      for (var i in _sites.values) {
        i.setIpfsCredentials(k1, k2, url);
      }

      return true;
    }
    return await _sites[site]!.setIpfsCredentials(k1, k2, '');
  }

  Future<bool> unfollowSite(String siteName, String url, String nick) async {
    if (siteName.isEmpty) {
      siteName = url.split('/')[2];
    }
    //todo flush caches etc
    _sites[siteName]!.terminate();
    _sites.remove(siteName);
    siteKeys.remove(siteName);
    if (homeSite == siteName) {
      homeSite = siteKeys.first;
    }

    ModeratorEntrySet _es = ModeratorEntrySet();
    _sites.forEach((key, value) {
      value.messages.all.forEach((element) {
        element.siteUrl = key;
        _es.add(element);
      });
    });
    _combinedEntrySetCache = _es;

    return true;
  }

  /*Future<bool> deleteEntry(String shortLink) async {
    if (_sites[homeSite] == null) return false;
    _sites[homeSite]!.deleteEntry(shortLink);
    return true;
  }*/

  Future<bool> dropEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = _sites[surl]!.dropEntry(shortLink);
    if (li == null) return false;
    _sites[surl]!.dropEntryOmni(shortLink);
    _tbsCache.remove(shortLink);
    return true;
  }

  bool markSeenOmni(String shortLink) {
    //hack to update cache
    if (_latestThreadsEntrySetCache.markSeenByShortLink(shortLink)) {
      ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
      if (target == null) return false;
      final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
      final li = _sites[surl]!.markSeenOmni(shortLink);
      if (li == null) return false;
    }

    _tbsCache.remove(shortLink);
    return true;
  }

  bool isSeenOmni(String shortLink) {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    return _sites[surl]!.isSeenOmni(shortLink);
  }

  bool peekThreadOmni(String shortLink) {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = _sites[surl]!.peekThreadOmni(shortLink);
    if (li == null) return false;
    return true;
  }

  Future<bool> muteAuthor(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    //TODO add OMNIdrops

    return await _sites[surl]!.muteAuthor(target);
  }

  Future<bool> shareEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final li = await _sites[surl]!.shareEntry(target);
    if (li == null) return false;
    return true;
  }

  Future<bool> likeEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final li = _sites[target.siteUrl]!.likeEntry(target);
    if (li == null) return false;
    return true;
  }

  Future<bool> unlikeEntry(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return false;
    final li = _sites[target.siteUrl]!.unlikeEntry(target);
    if (li == null) return false;
    return true;
  }

  Future<ModeratorEntry?> postEntry(
      String shortLink, String text, Uint8List attachment) async {
    if (_sites[homeSite] == null) return null;

    final pe = await _sites[homeSite]!
        .newThread(homeSite, text, attachment, '', shortLink, '');
    //no katamari is trouble
    if (pe == null) return null;

    if (pe != null && pe.isEmpty) return null;
    return pe!.first;
  }

  Future<List<ModeratorEntry>?> replyEntry(String shortLink, String replyText,
      String ppShortLink, Uint8List attachement) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    if (target == null) return null;
    final surl = target.siteUrl.isNotEmpty ? target.siteUrl : homeSite;

    return _sites[surl]!
        .newReply(replyText, target.shortLink, ppShortLink, attachement)
        .then((value) {
      if (value != null && value!.isNotEmpty) {
        _tbsCache.remove(shortLink);
      }
      return value;
    });
  }

  Future<List<ModeratorEntry>?> refreshThread(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    //if thread missing try to find on serv
    //if (target == null) return false;
    //final surl = target?.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final surl = homeSite;
    final r = await _sites[surl]!.refreshThreadViaHTTP(shortLink);
    if (r == null) return r;
    _tbsCache.clear();
    return r;
  }

  Future<List<ModeratorEntry>?> refreshTagged(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    //if thread missing try to find on serv
    //if (target == null) return false;
    //final surl = target?.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final surl = homeSite;
    final r = await _sites[surl]!.refreshTaggedViaHTTP(shortLink);
    if (r == null) return r;
    _tbsCache.clear();
    return r;
  }

  Future<List<ModeratorEntry>?> refreshAuthor(String shortLink) async {
    ModeratorEntry? target = findModeratorEntryByShortLink(shortLink);
    //if thread missing try to find on serv
    //if (target == null) return false;
    //final surl = target?.siteUrl.isNotEmpty ? target.siteUrl : homeSite;
    final surl = homeSite;
    final r = await _sites[surl]!.refreshAuthorViaHTTP(shortLink);
    if (r == null) return r;
    _tbsCache.clear();
    return r;
  }

  Future<Uint8List> pullIPFScid(String cid, int xx) async {
    if (cid.isEmpty) {
      return Uint8List(0);
    }
    return _sites[homeSite]!.xxC.pullIPFScid(cid, xx);
  }

  //unstuck ipfs queue
  int get ipfsQflushFailed => _sites[homeSite]!.ipfsQflushFailed;

  ModeratorEntry? findModeratorEntryByShortLink(String shortLink) {
    for (var i in _sites.values) {
      final t = i.findModeratorEntryByShortLink(shortLink);
      if (t != null) return t;
    }
    return null;
  }

  Future<bool> get refreshAllViaHTTP async {
    allSites.forEach((element) async {
      await element.refreshViaHTTP();
    });

    return true;
  }

  List<int> get stats {
    final s = [
      0, //IOreceived
      0, //IOsent
      0,
      0,
      0, //memUsed
      0, //audience
      0, 0, 0, 0, 0, 0, 0
    ];
    //combined stats for each followed server
    allSites.forEach((element) async {
      var ii = 0;
      for (var i in element.stats) {
        s[ii] = s[ii] + i;
        ii++;
      }
    });
    return s;
  }

  bool online = true;
  void setOnline() {
    online = true;
    allSites.map((e) => e.setOnline);
  }

  void setOffline() {
    online = false;
    allSites.map((e) => e.setOffline);
  }

  Uint8List get katamari {
    return _sites[homeSite]!.katamari;
  }
}
